﻿'use strict';

var chatMessenger = {};
$(function () {
    if (chatUtils.getIEVersion() === 0) {
        var messengerSessionsSpacerElement = document.getElementById("imaginaryMessengerSession");
        new IntersectionObserver(function (entries, observer) {
            if (entries.length === 1) {
                let entry = entries[0];
                if (entry.intersectionRatio === 0 && !entry.isIntersecting)
                    hideLeastCurrentMesengers();
                else if (entry.intersectionRatio === 1 && entry.isIntersecting)
                    showMostCurrentMessengers();
            }
        }, {
                root: null,
                rootMargin: '0px',
                threshold: [0.0, 1.0]
            }).observe(messengerSessionsSpacerElement);
    }

    $("#messengerWidget > .messenger-button ul.dropdown-menu, #messengerWidget > .messenger-content > .messenger-widget-head > .messenger-status ul.dropdown-menu").click(function (e) {
        $(this).parent().find("[data-toggle=dropdown]").dropdown('toggle');
        e.stopPropagation();
    });

    $("#messengerWidget #messengerSettings").click(function (e) {
        $("#dialog-messengerSettings").dialog('open');
        e.stopPropagation();
    });

    function initSettingsDialog(userProviderKey) {
        $("#dialog-messengerSettings").dialog({
            autoOpen: false,
            resizable: false,
            width: "auto",
            height: "auto",
            modal: true,
            buttons: [{
                text: Localization_Save,
                click: function () {
                    var self = this;
                    $(self).find("div.loading-32").show();
                    $(document.getElementById('chatMessengerSettings')).load(
                        RQ_URL + "/Chat/PartialChatMessengerSettings",
                        {
                            UserProviderKey: userProviderKey,
                            Minimization: !$("#messengerMinimization").is(":checked"),
                            Sounds: $("#messengerSounds").is(":checked")
                        }, function (responseText, textStatus, jqXHR) {
                            if (textStatus === "success") {
                                initSettingsDialog(userProviderKey);
                            }

                            $(self).dialog("close");
                        }
                    );
                }
            },
            {
                text: Localization_Close,
                click: function () {
                    $(this).dialog("close");
                }
            }
            ],
            close: function () {
                $("#dialog-messengerSettings").find("div.loading-32").hide();
            }
        });
    }

    function loadMessengerSessionTime(id) {
        if (!('sessionStorage' in window))
            return;

        let res = Number(sessionStorage.getItem('messengerSessionTime_' + id));
        return isNaN(res) ? 0 : res;
    }

    function saveMessengerSessionTime(id, time) {
        if (typeof time !== 'number')
            throw "Parameter 'visible' is not a boolean.";

        if (!('sessionStorage' in window))
            return;

        return sessionStorage.setItem('messengerSessionTime_' + id, time);
    }

    function loadMessengerFragmentIsMaximized(id) {
        if (!('sessionStorage' in window))
            return;

        return sessionStorage.getItem('messengerFragmentIsMaximized_' + id) === 'true';
    }

    function saveMessengerFragmentIsMaximized(id, maximized) {
        if (typeof maximized !== 'boolean')
            throw "Parameter 'maximized' is not a boolean.";

        if (!('sessionStorage' in window))
            return;

        return sessionStorage.setItem('messengerFragmentIsMaximized_' + id, maximized);
    }

    function processDisplayingOfSession($session) {
        let time = new Date().getTime();
        saveMessengerSessionTime($session.attr('id'), time);
        $session.closest(".messenger-session").find("#currenticity").val(time);
        $session.show();
    }

    function subscribeViewModel(newViewModel) {
        initSettingsDialog(newViewModel.userProviderKey);

        function toggleMessengerWidget() {
            $("#messengerWidget #messengerButton, #messengerWidget #messengerContent").toggle();
            chatMessenger.toggleMessengerFragment({
                show: "#messengerWidget #messengerContent",
                hide: "#messengerWidget #messengerButton",
                revert: $("#messengerWidget #messengerButton").is(":visible")
            });

            if ($("#messengerWidget #messengerContent").is(":visible")) {
                newViewModel.getVirtualMessengerSessions(true);
                newViewModel.getUsers();
            }
        }

        $("#messengerWidget #messengerButton, #messengerWidget .messenger-widget-head").click(function (e) {
            toggleMessengerWidget();
        });

        function displayMessengerWebNotification(chatMessage) {
            function html2text(html) {
                var tag = document.createElement('div');
                tag.innerHTML = html;
                return tag.innerText;
            }

            if ("Notification" in window && Notification.permission === "granted") {
                var hidden;
                if (typeof document.hidden !== "undefined") { // Opera 12.10 and Firefox 18 and later support 
                    hidden = "hidden";
                } else if (typeof document.msHidden !== "undefined") {
                    hidden = "msHidden";
                } else if (typeof document.webkitHidden !== "undefined") {
                    hidden = "webkitHidden";
                }

                if (hidden !== undefined && (chatMessage.Type === chatUtils.ChatMessageTypeEnum.COMMON ||
                    chatMessage.Type === chatUtils.ChatMessageTypeEnum.ATTACHMENTS ||
                    chatMessage.Type === chatUtils.ChatMessageTypeEnum.START_CALL)) {
                    let faviconLink = document.querySelector("link[rel*='icon']");
                    let notification = new Notification("{0} {1}\u2026".format(chatMessage.FullName, Localization_Chat_Wrote), {
                        body: chatMessage.Type === chatUtils.ChatMessageTypeEnum.START_CALL ? Localization_Chat_Call : html2text(chatMessage.Text),
                        icon: faviconLink.href,
                        badge: faviconLink.href,
                        silent: true,
                        requireInteraction: true,
                        timestamp: new Date(chatMessage.Date),
                        tag: chatMessage.SessionId,
                        renotify: true
                    });

                    notification.onclick = function (event) {
                        notification.close();
                        window.focus();

                        $.each(newViewModel.chatItems(), function (index, chatItem) {
                            if (chatItem.sessionId === chatMessage.SessionId) {
                                chatItem.isSelected(true);
                                return false;
                            }
                        });
                    };
                }
            }
        }

        function messageAdded(message, chatItem) {
            if (chatItem instanceof ChatMessengerItem) {
                if (message.Type !== chatUtils.ChatMessageTypeEnum.COMMON_INCOMPLETE && message.UserProviderKey !== newViewModel.userProviderKey && !message.PostLoaded) {
                    var $sessionButton = $("#" + chatItem.sessionId).find(".messenger-session-button");
                    if ($("#chatMessengerMinimization").val() === "False" && $sessionButton.is(":visible")) {
                        toggleMessengerFragment({
                            parent: '.messenger-session',
                            show: '.messenger-session-head, .messenger-session-body',
                            hide: '.messenger-session-button',
                            revert: false
                        }, $sessionButton);
                    }

                    processDisplayingOfSession($("#" + chatItem.sessionId));

                    if ($("#chatMessengerMessageReceivedNotification").val() === "True" &&
                        (message.Type !== chatUtils.ChatMessageTypeEnum.COMMON || message.Type !== chatUtils.ChatMessageTypeEnum.ATTACHMENTS || message.Type !== chatUtils.ChatMessageTypeEnum.AUTOMATION)) {
                        displayMessengerWebNotification(message);
                    }
                    
                    var soundElement = document.getElementById("chatMessengerSound");
                    if (soundElement !== null) {
                        soundElement.currentTime = 0;
                        soundElement.volume = 0.7;
                        soundElement.play();
                    }
                }
            }
        }

        function subscribeChatItemMessages(chatItem) {
            return chatItem.viewModel().messages.subscribe(function (changes) {
                changes.forEach(function (change) {
                    if (change.status === 'added') {
                        messageAdded(change.value, chatItem);
                    }
                });
            }, null, "arrayChange");
        }

        var chatItemsSubscriptions = {};
        newViewModel.chatItems.subscribe(function (changes) {
            changes.forEach(function (change) {
                let chatItem = change.value;
                if (chatItem instanceof ChatMessengerItem) {
                    if (change.status === 'added') {
                        chatItemsSubscriptions[chatItem.sessionId] = subscribeChatItemMessages(chatItem);
                    }
                    else if (change.status === 'deleted') {
                        if (chatItemsSubscriptions[chatItem.sessionId]) {
                            chatItemsSubscriptions[chatItem.sessionId].dispose();
                            delete chatItemsSubscriptions[chatItem.sessionId];
                        }
                        else
                            console.error(chatItemsSubscriptions);
                    }
                }
            });

        }, null, "arrayChange");

        let autoAwayTimeout = Number(document.getElementById("chatMessengerAutoAwayTimeoutSec").value);
        if (autoAwayTimeout > 0) {
            autoAwayTimeout *= 1000;
            new Idle({
                //onHidden: onHiddenCallback,
                //onVisible: onVisibleCallback,
                onAway: function () { newViewModel.awayCallback(autoAwayTimeout); },
                onAwayBack: newViewModel.awayBackCallback,
                awayTimeout: autoAwayTimeout
            });
        }
    }

    function toggleMessengerFragment(selectors, $childElement) {
        let showSelector = selectors.revert ? selectors.hide : selectors.show;
        let hideSelector = selectors.revert ? selectors.show : selectors.hide;
        let $parent = $childElement ? $childElement.closest(selectors.parent) : null;
        let $show = $parent ? $parent.find(showSelector) : $(showSelector);
        let $hide = $parent ? $parent.find(hideSelector) : $(hideSelector);

        $show.show();
        $hide.hide();

        if ($childElement && selectors.revert) {
            let chatMessengerSession = ko.dataFor($childElement[0]);
            if (!(chatMessengerSession instanceof ChatMessengerSessionViewModel))
                throw "chatMessengerSession is not a view model.";

            toggleSearching(chatMessengerSession, true);
        }

        if (!$parent) {
            var idArray = [$show.attr('id'), $hide.attr('id')];
            if (selectors.revert)
                idArray.reverse();
        }

        saveMessengerFragmentIsMaximized($parent ? $parent.attr('id') : idArray.join("_"), selectors.revert ? false : true);
    }

    function searchMessages(chatMessengerSession) {
        if (!(chatMessengerSession instanceof ChatMessengerSessionViewModel))
            throw "chatMessengerSession is not a view model.";

        if (chatMessengerSession.expressionToSearch() && chatMessengerSession.expressionToSearch().length > 0) {
            chatMessengerSession.searchMessages();

            let $messenger = $('#' + chatMessengerSession.sessionId);
            let $searching = $messenger.find('a[href^="#tabSearchingContainer_"]');
            $searching.tab('show');
            $('div[id^="tabSearchingContainer_"] > .messenger-searching-container .messenger-searching-box input[type="text"]').focus();
        }
    }

    function toggleSearching(chatMessengerSession, forceHide) {
        if (!(chatMessengerSession instanceof ChatMessengerSessionViewModel))
            throw "chatMessengerSession is not a view model.";

        let $messenger = $('#' + chatMessengerSession.sessionId);
        let $messagesLink = $messenger.find("a[href^='#tabMessagesContainer_']");
        let $messagesPane = $messenger.find("div[id^='tabMessagesContainer_']");
        let $searchingBox = $messenger.find('.messenger-searching-box');

        chatMessengerSession.resetSearching();
        if (forceHide)
            $searchingBox.hide();
        else {
            $searchingBox.toggle();
            $searchingBox.each(function (index, element) {
                let $element = $(element);
                if (!$element.is(":hidden")) {
                    $(element).find('input[type="text"]').focus();
                }
            });
        }

        if (!$messagesPane.hasClass('active'))
            $messagesLink.tab('show');
    }

    function isElementXPercentInViewport(el, percentVisible) {
        let
            rect = el.getBoundingClientRect(),
            windowWidth = (window.innerWidth || document.documentElement.clientWidth);

        return !(
            Math.floor(100 - (((rect.left >= 0 ? 0 : rect.left) / +-(rect.width / 1)) * 100)) < percentVisible ||
            Math.floor(100 - ((rect.right - windowWidth) / rect.width) * 100) < percentVisible
        );
    }

    function hideLeastCurrentMesengers() {
        $("#messengerContainer > .messenger-session").sort(function (s1, s2) {
            let $s1 = $(s1), $s2 = $(s2);
            let isSelected1 = $s1.find('#chatMessengerIsSelected').val() === 'true', isSelected2 = $s2.find('#chatMessengerIsSelected').val() === 'true';

            if (isSelected1)
                return 1;
            if (isSelected2)
                return -1;

            let currenticityTicks1 = Number($s1.find("#currenticity").val()), currenticityTicks2 = Number($s2.find("#currenticity").val());
            let res = currenticityTicks1 - currenticityTicks2;

            return res !== 0 ? res : $s2.index() - $s1.index();
        }).each(function () {
            let $session = $(this);
            if ($session.is(":visible")) {
                $session.hide();
                if (isElementXPercentInViewport(messengerSessionsSpacerElement, 0))
                    return false;
            }
        });
    }

    function showMostCurrentMessengers() {
        $("#messengerContainer > .messenger-session").sort(function (s1, s2) {
            let $s1 = $(s1), $s2 = $(s2);
            let isSelected1 = $s1.find('#chatMessengerIsSelected').val() === 'true', isSelected2 = $s2.find('#chatMessengerIsSelected').val() === 'true';

            if (isSelected1)
                return -1;
            if (isSelected2)
                return 1;

            let currenticityTicks1 = Number($s1.find("#currenticity").val()), currenticityTicks2 = Number($s2.find("#currenticity").val());
            let res = currenticityTicks2 - currenticityTicks1;

            return res !== 0 ? res : $s1.index() - $s2.index();
        }).each(function () {
            let $session = $(this);
            if (!$session.is(":visible")) {
                $session.show();
                if (!isElementXPercentInViewport(messengerSessionsSpacerElement, 100))
                    return false;
            }
        });
    }

    function initCustomBindings() {
        ko.bindingHandlers.clickToggleVisibility = {
            init: function (element, valueAccessor, allBindingsAccessor, viewModel) {
                $(element).click(function (e) {
                    toggleMessengerFragment(ko.unwrap(valueAccessor()), $(this));
                });
            }
        };

        ko.bindingHandlers.needFocus = {
            init: function (element, valueAccessor, allBindings, viewModel) {
                let $element = $(element);

                $element.focus(function () {
                    valueAccessor()(true);
                });

                $element.blur(function () {
                    valueAccessor()(false);
                });
            },
            update: function (element, valueAccessor, allBindings, viewModel) {
                let $element = $(element);
                if (ko.unwrap(valueAccessor())) {
                    if (!$element.is(":focus")) {
                        setTimeout(function () {
                            toggleMessengerFragment({
                                parent: '.messenger-session',
                                show: '.messenger-session-head, .messenger-session-body',
                                hide: '.messenger-session-button',
                                revert: false
                            }, $element);
                            $element.closest(".messenger-session").show();
                            $element.focus(); // async focus kvůli nově otevřenému messengeru
                        }, 0);

                        processDisplayingOfSession($element.closest(".messenger-session"));
                        $element.focus(); //sync focus kvůli již otevřeným messengerům
                    }
                }
                else {
                    if ($element.is(":focus"))
                        $element.blur();
                }
            }
        };
    }

    //ready
    chatMessenger.subscribeViewModel = subscribeViewModel;
    chatMessenger.loadMessengerFragmentIsMaximized = loadMessengerFragmentIsMaximized;
    chatMessenger.loadMessengerSessionTime = loadMessengerSessionTime;
    chatMessenger.toggleMessengerFragment = toggleMessengerFragment;
    chatMessenger.searchMessages = searchMessages;
    chatMessenger.toggleSearching = toggleSearching;
    initCustomBindings();
});